package org.rcisoft.app.appMemInfo.controller;


/*固定导入*/

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.drew.imaging.ImageMetadataReader;
import com.drew.metadata.Metadata;
import com.drew.metadata.exif.ExifIFD0Directory;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.media.Schema;
import org.rcisoft.business.memInfo.entity.*;
import org.rcisoft.business.memInfo.service.MemInfoService;
import org.rcisoft.core.anno.CyEncryptSm4Anno;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.component.CyGlobal;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.result.enums.CyResSysExcEnum;
import org.rcisoft.core.service.CyFileStorageService;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


import javax.imageio.ImageIO;
import javax.validation.Valid;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Arrays;

/**
* Created by cy on 2024年12月20日 下午3:35:34.
*/
@RestController
@RequestMapping("/app")
public class AppMemInfoController extends CyPaginationController<MemInfo> {


    @Autowired
    private MemInfoService memInfoServiceImpl;

    @Autowired
    private CyGlobal cyGlobal;

    @Autowired
    private CyFileStorageService cyFileStorageService;

    @CyOpeLogAnno(title = "system-会员表管理-新增会员表", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="添加会员表", description="添加会员表")
    @PostMapping(value = "/open/memInfo/add")
    @CyEncryptSm4Anno
    public CyResult add( MemInfo memInfo) {
        CyPersistModel data = memInfoServiceImpl.persist(memInfo);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        memInfo);
    }

    @CyOpeLogAnno(title = "system-会员表管理-校验手机号", businessType = CyLogTypeEnum.OTHER)
    @Operation(summary="校验手机号", description="校验手机号")
    @PostMapping(value = "/open/memInfo/checkInfo")
    @CyEncryptSm4Anno
    public CyResult checkInfo(@Valid MemInfo memInfo) {
        return CyResultGenUtil.genSuccessResult(memInfoServiceImpl.checkInfo(memInfo));
    }


    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询单一会员表", description="查询单一会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @GetMapping("/open/memInfo/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        MemInfo info = memInfoServiceImpl.findById(businessId);
        info.setMemWxCode(null);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                info);
    }

    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询会员表集合", description="分页查询会员表集合")
    @GetMapping(value = "/open/memInfo/list")
    public CyGridModel listByPagination(MemInfo memInfo) {
        IPage<MemInfo> memInfoIPage = memInfoServiceImpl.findAllByPagination(getPaginationUtility(), memInfo);
        for (MemInfo record : memInfoIPage.getRecords()) {
            record.setMemWxCode(null);
        }
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询喜欢集合", description="分页查询喜欢集合")
    @GetMapping(value = "/open/memInfo/likeList")
    public CyGridModel likeListByPagination(MemInfo memInfo) {
        IPage<MemInfo> memInfoIPage = memInfoServiceImpl.findLikeListByPagination(getPaginationUtility(), memInfo);
        for (MemInfo record : memInfoIPage.getRecords()) {
            record.setMemWxCode(null);
        }
        return getGridModelResponse();
    }

    /**
     * 文件上传new （2.0）
     *
     * @param file
     * @return
     */
    @Operation(summary = "文件上传", description = "文件上传")
    @Parameters({@Parameter(name = "file", description = "文件", required = true)})
    @PostMapping(value = "/open/fileUpload")
    @ResponseBody
    public CyResult fileUpload(@RequestParam("file") MultipartFile file, @RequestParam("temp") String temp, @RequestParam(name = "bucket", required = false) String bucket,
                               @RequestParam(name = "isCompacted", required = false,defaultValue = "false") boolean isCompacted) {
        //文件上传限制
        String[] split = cyGlobal.getFileUploadFormatRestrictions().split(",");
        if (file==null || Arrays.stream(split).anyMatch(file.getOriginalFilename()::contains)){
            throw new CyServiceException(CyResSysExcEnum.FILE_STORAGE_ERROR);
        }
        //如果上传文件是大于1Mb的图片，判断是否需要压缩
        String[] imageExtensions = {".jpg", ".jpeg", ".png", ".gif", ".bmp", ".tiff", ".webp"};
        long maxSize = 1024 * 1024;  // 1MB
        if (Arrays.stream(imageExtensions).anyMatch(file.getOriginalFilename()::contains) && file.getSize() > maxSize && isCompacted) {
            file = compressImage(file);
        }
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cyFileStorageService.filePath(file, temp, bucket));
    }

    //图片压缩
    private static MultipartFile compressImage(MultipartFile file) {
        BufferedImage image = null;
        try {
            image = ImageIO.read(file.getInputStream());
            // 获取图像的方向信息并进行旋转调整
            image = fixImageOrientation(file, image);
//            // 设置压缩质量
//            float quality =  1.0f;
            // 调整图片尺寸，设置目标宽度和高度
            int targetWidth = 800; // 目标宽度（可以根据需求调整）
            int targetHeight = 600; // 目标高度（可以根据需求调整）

            // 使用Imgscalr进行图像压缩（缩放）
//            BufferedImage compressedImage = Scalr.resize(image,Scalr.Method.QUALITY, Scalr.Mode.AUTOMATIC,500);

            // 将BufferedImage写入ByteArrayOutputStream
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            String originalFilename = file.getOriginalFilename();
            String extension = originalFilename.substring(originalFilename.lastIndexOf('.') + 1).toLowerCase();
            ImageIO.write(image, extension, outputStream); // 假设原始图片格式为jpg


            // 将压缩后的图片数据转换为byte数组
            byte[] compressedImageData = outputStream.toByteArray();

            // 将byte数组转换为MultipartFile
            return new MockMultipartFile(file.getName(), originalFilename,
                    file.getContentType(), compressedImageData);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return file;

    }

    /**
     * 获取图像的 EXIF 方向标签
     */
    private static int getExifOrientation(MultipartFile file) {
        try {
            Metadata metadata = ImageMetadataReader.readMetadata(file.getInputStream());
            ExifIFD0Directory directory = metadata.getFirstDirectoryOfType(ExifIFD0Directory.class);

            if (directory != null) {
                Integer  orientationTag = directory.getInteger(ExifIFD0Directory.TAG_ORIENTATION);
                if (orientationTag != null) {
                    return orientationTag;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return 1;  // 默认方向，无旋转
    }

    /**
     * 旋转图像
     */
    private static BufferedImage rotateImage(BufferedImage image, int angle) {
        int width = image.getWidth();
        int height = image.getHeight();

        // 计算旋转后的新宽高
        double radians = Math.toRadians(angle);
        int newWidth = (int) Math.abs(width * Math.cos(radians)) + (int) Math.abs(height * Math.sin(radians));
        int newHeight = (int) Math.abs(height * Math.cos(radians)) + (int) Math.abs(width * Math.sin(radians));

        // 创建一个新的图像，背景色为白色或透明
        BufferedImage rotatedImage = new BufferedImage(newWidth, newHeight, image.getType());
        Graphics2D g2d = rotatedImage.createGraphics();
//        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
//        g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
//
//        // 设置旋转中心
        g2d.translate((newWidth - width) / 2, (newHeight - height) / 2);  // 居中
//        g2d.rotate(radians, width / 2, height / 2);  // 旋转
        g2d.rotate(radians, width / 2, height / 2);

        // 绘制原图像
        g2d.drawImage(image, 0, 0, null);
        g2d.dispose();

        return rotatedImage;
    }

    /**
     * 根据 EXIF 信息修正图像方向
     */
    private static BufferedImage fixImageOrientation(MultipartFile file, BufferedImage image) throws IOException {
        int orientation = getExifOrientation(file);

        switch (orientation) {
            case 3:
                return rotateImage(image, 180);
            case 6:
                return rotateImage(image, 90);
            case 8:
                return rotateImage(image, 270);
            default:
                return image;  // 无需旋转
        }
    }

    @PreAuthorize("@cyPerm.hasPerm('app:mem:query')")
    @CyOpeLogAnno(title = "system-会员表管理-校验实名认证信息", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="校验实名认证信息", description="校验实名认证信息")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false,  schema = @Schema(type = "string"))})
    @PutMapping("/memInfo/checkIdCard/{userId:\\w+}")
    @CyEncryptSm4Anno
    public CyResult checkIdCard(@PathVariable int userId, @RequestBody MemInfo memInfo, BindingResult bindingResult) {
        memInfo.setUserId(userId);
        CyPersistModel data = memInfoServiceImpl.checkIdCard(memInfo);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                memInfo);
    }

    /**
     * 会员基础信息页面-修改个人资料
     */
    @PreAuthorize("@cyPerm.hasPerm('app:mem:update')")
    @CyOpeLogAnno(title = "system-会员表管理-修改会员表", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="修改会员表", description="修改会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false,  schema = @Schema(type = "string"))})
    @PutMapping("/memInfo/update")
    @CyEncryptSm4Anno
    public CyResult update( @RequestBody MemInfo memInfo) {
        CyPersistModel data = memInfoServiceImpl.update(memInfo);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                memInfo);
    }

    /**
     * 会员详情页面-关注
     */
    @PreAuthorize("@cyPerm.hasPerm('app:mem:follow')")
    @CyOpeLogAnno(title = "system-会员关注-会员关注", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="会员关注", description="会员关注")
    @PostMapping(value = "/memInfo/follow")
    @CyEncryptSm4Anno
    public CyResult follow( @RequestBody MemFollowDTO followDTO) {
        CyPersistModel data = memInfoServiceImpl.addFollow(followDTO);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                followDTO);
    }

    /**
     * 会员详情页面-查看是否关注
     */
    @PreAuthorize("@cyPerm.hasPerm('app:mem:query')")
    @CyOpeLogAnno(title = "system-会员关注-查询是否关注", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询是否关注", description="查询是否关注")
    @GetMapping(value = "/memInfo/isFollow")
    @CyEncryptSm4Anno
    public CyResult isFollow(MemFollowDTO followDTO) {
        MemFollowDTO follow = memInfoServiceImpl.isFollow(followDTO);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                follow);
    }

    /**
     * 会员详情页面-取消关注
     */
    @PreAuthorize("@cyPerm.hasPerm('app:mem:follow')")
    @CyOpeLogAnno(title = "system-会员取消关注-会员取消关注", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="会员取消关注", description="会员取消关注")
    @PostMapping(value = "/memInfo/followDelete")
    @CyEncryptSm4Anno
    public CyResult followDelete( @RequestBody MemFollowDTO followDTO) {
        CyPersistModel data = memInfoServiceImpl.deleteFollow(followDTO);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                followDTO);
    }

    /**
     * 会员详情页面-点赞
     */
    @PreAuthorize("@cyPerm.hasPerm('app:mem:like')")
    @CyOpeLogAnno(title = "system-会员点赞-会员点赞", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="会员点赞", description="会员点赞")
    @PostMapping(value = "/memInfo/like")
    @CyEncryptSm4Anno
    public CyResult like( @RequestBody MemLikeDTO likeDTO) {
        CyPersistModel data = memInfoServiceImpl.addLike(likeDTO);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                likeDTO);
    }

    /**
     * 排行榜
     */
    @CyOpeLogAnno(title = "system-会员表管理-查询排行榜", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询排行榜集合", description="分页查询排行榜集合")
    @GetMapping(value = "/open/memInfo/rank")
    public CyGridModel rankByPagination(MemInfo memInfo) {
        IPage<MemInfo> memInfoIPage = memInfoServiceImpl.findRankByPagination(getPaginationUtility(), memInfo);
        for (MemInfo record : memInfoIPage.getRecords()) {
            record.setMemWxCode(null);
        }
        return getGridModelResponse();
    }
}
